@@ -10,61 +10,15 @@ from collections import namedtuple
 from contextlib import contextmanager
 
 from . import fast_data_types as defines
-from .config_utils import (
+from .conf.definition import as_conf_file, config_lines
+from .conf.utils import (
     init_config, key_func, load_config as _load_config, merge_dicts,
-    parse_config_base, positive_float, positive_int, python_string, to_bool,
-    to_cmdline, to_color, unit_float
+    parse_config_base, python_string, to_bool, to_cmdline
 )
+from .config_data import all_options, parse_mods, type_map
 from .constants import cache_dir, defconf
-from .fast_data_types import CURSOR_BEAM, CURSOR_BLOCK, CURSOR_UNDERLINE
-from .layout import all_layouts
-from .rgb import color_as_int, color_from_int
 from .utils import log_error
 
-MINIMUM_FONT_SIZE = 4
-
-
-def to_font_size(x):
-    return max(MINIMUM_FONT_SIZE, float(x))
-
-
-cshapes = {
-    'block': CURSOR_BLOCK,
-    'beam': CURSOR_BEAM,
-    'underline': CURSOR_UNDERLINE
-}
-
-
-def to_cursor_shape(x):
-    try:
-        return cshapes[x.lower()]
-    except KeyError:
-        raise ValueError(
-            'Invalid cursor shape: {} allowed values are {}'.format(
-                x, ', '.join(cshapes)
-            )
-        )
-
-
-mod_map = {'CTRL': 'CONTROL', 'CMD': 'SUPER', '⌘': 'SUPER', '⌥': 'ALT', 'OPTION': 'ALT', 'KITTY_MOD': 'KITTY'}
-
-
-def parse_mods(parts, sc):
-
-    def map_mod(m):
-        return mod_map.get(m, m)
-
-    mods = 0
-    for m in parts:
-        try:
-            mods |= getattr(defines, 'GLFW_MOD_' + map_mod(m.upper()))
-        except AttributeError:
-            log_error('Shortcut: {} has unknown modifier, ignoring'.format(sc))
-            return
-
-    return mods
-
-
 named_keys = {
     "'": 'APOSTROPHE',
     ',': 'COMMA',
@@ -149,6 +103,22 @@ def float_parse(func, rest):
     return func, (float(rest),)
 
 
+@func_with_args('change_font_size')
+def parse_change_font_size(func, rest):
+    vals = rest.split(' ', 1)
+    if len(vals) != 2:
+        log_error('Invalid change_font_size specification: {}, treating it as default'.format(rest))
+        args = [True, None, 0]
+    else:
+        args = [vals[0].lower() == 'all', None, 0]
+        amt = vals[1]
+        if amt[0] in '+-':
+            args[1] = amt[0]
+            amt = amt[1:]
+        args[2] = float(amt)
+    return func, args
+
+
 def parse_key_action(action):
     parts = action.split(' ', 1)
     func = parts[0]
@@ -273,149 +243,6 @@ def parse_send_text(val, key_definitions):
     return parse_key(key_str, key_definitions)
 
 
-def to_modifiers(val):
-    return parse_mods(val.split('+'), val) or 0
-
-
-def uniq(vals, result_type=list):
-    seen = set()
-    seen_add = seen.add
-    return result_type(x for x in vals if x not in seen and not seen_add(x))
-
-
-def to_layout_names(raw):
-    parts = [x.strip().lower() for x in raw.split(',')]
-    ans = []
-    for p in parts:
-        if p == '*':
-            ans.extend(sorted(all_layouts))
-            continue
-        name = p.partition(':')[0]
-        if name not in all_layouts:
-            raise ValueError('The window layout {} is unknown'.format(p))
-        ans.append(p)
-    return uniq(ans)
-
-
-def adjust_line_height(x):
-    if x.endswith('%'):
-        return float(x[:-1].strip()) / 100.0
-    return int(x)
-
-
-def macos_titlebar_color(x):
-    x = x.strip('"')
-    if x == 'system':
-        return 0
-    if x == 'background':
-        return 1
-    return (color_as_int(to_color(x)) << 8) | 2
-
-
-def box_drawing_scale(x):
-    ans = tuple(float(x.strip()) for x in x.split(','))
-    if len(ans) != 4:
-        raise ValueError('Invalid box_drawing scale, must have four entries')
-    return ans
-
-
-def tab_separator(x):
-    for q in '\'"':
-        if x.startswith(q) and x.endswith(q):
-            x = x[1:-1]
-            break
-    if not x.strip():
-        x = ('\xa0' * len(x)) if x else defaults.tab_separator
-    return x
-
-
-def tab_font_style(x):
-    return {
-        'bold-italic': (True, True),
-        'bold': (True, False),
-        'italic': (False, True)
-    }.get(x.lower().replace('_', '-'), (False, False))
-
-
-def tab_bar_edge(x):
-    return {'top': 1, 'bottom': 3}.get(x.lower(), 3)
-
-
-def url_style(x):
-    return url_style.map.get(x, url_style.map['curly'])
-
-
-url_style.map = dict(
-    ((v, i) for i, v in enumerate('none single double curly'.split()))
-)
-
-type_map = {
-    'allow_remote_control': to_bool,
-    'adjust_line_height': adjust_line_height,
-    'adjust_column_width': adjust_line_height,
-    'scrollback_lines': positive_int,
-    'scrollback_pager': to_cmdline,
-    'open_url_with': to_cmdline,
-    'font_size': to_font_size,
-    'font_size_delta': positive_float,
-    'focus_follows_mouse': to_bool,
-    'cursor_shape': to_cursor_shape,
-    'open_url_modifiers': to_modifiers,
-    'rectangle_select_modifiers': to_modifiers,
-    'repaint_delay': positive_int,
-    'input_delay': positive_int,
-    'sync_to_monitor': to_bool,
-    'close_on_child_death': to_bool,
-    'window_border_width': positive_float,
-    'window_margin_width': positive_float,
-    'window_padding_width': positive_float,
-    'wheel_scroll_multiplier': float,
-    'visual_bell_duration': positive_float,
-    'enable_audio_bell': to_bool,
-    'click_interval': positive_float,
-    'mouse_hide_wait': positive_float,
-    'cursor_blink_interval': positive_float,
-    'cursor_stop_blinking_after': positive_float,
-    'enabled_layouts': to_layout_names,
-    'remember_window_size': to_bool,
-    'initial_window_width': positive_int,
-    'initial_window_height': positive_int,
-    'macos_hide_titlebar': to_bool,
-    'macos_hide_from_tasks': to_bool,
-    'macos_option_as_alt': to_bool,
-    'macos_titlebar_color': macos_titlebar_color,
-    'box_drawing_scale': box_drawing_scale,
-    'dynamic_background_opacity': to_bool,
-    'background_opacity': unit_float,
-    'dim_opacity': unit_float,
-    'tab_separator': tab_separator,
-    'active_tab_font_style': tab_font_style,
-    'inactive_tab_font_style': tab_font_style,
-    'inactive_text_alpha': unit_float,
-    'url_style': url_style,
-    'copy_on_select': to_bool,
-    'window_alert_on_bell': to_bool,
-    'tab_bar_edge': tab_bar_edge,
-    'bell_on_tab': to_bool,
-    'kitty_mod': to_modifiers,
-    'clear_all_shortcuts': to_bool,
-    'clipboard_control': lambda x: frozenset(x.lower().split()),
-    'window_resize_step_cells': int,
-    'window_resize_step_lines': int,
-}
-
-for name in (
-    'foreground background cursor active_border_color inactive_border_color'
-    ' selection_foreground selection_background url_color bell_border_color'
-).split():
-    type_map[name] = to_color
-for i in range(256):
-    type_map['color{}'.format(i)] = to_color
-for a in ('active', 'inactive'):
-    for b in ('foreground', 'background'):
-        type_map['%s_tab_%s' % (a, b)] = to_color
-
-
 def special_handling(key, val, ans):
     if key == 'map':
         parse_key(val, ans['key_definitions'])
@@ -434,9 +261,6 @@ def special_handling(key, val, ans):
 
 
 defaults = None
-default_config_path = os.path.join(
-    os.path.dirname(os.path.abspath(__file__)), 'kitty.conf'
-)
 
 
 def parse_config(lines, check_keys=True):
@@ -454,15 +278,10 @@ def parse_config(lines, check_keys=True):
 
 def parse_defaults(lines, check_keys=False):
     ans = parse_config(lines, check_keys)
-    dfctl = defines.default_color_table()
-
-    for i in range(16, 256):
-        k = 'color{}'.format(i)
-        ans.setdefault(k, color_from_int(dfctl[i]))
     return ans
 
 
-Options, defaults = init_config(default_config_path, parse_defaults)
+Options, defaults = init_config(config_lines(all_options), parse_defaults)
 actions = frozenset(all_key_actions) | frozenset(
     'run_simple_kitten combine send_text goto_tab goto_layout set_font_size new_tab_with_cwd new_window_with_cwd new_os_window_with_cwd'.
     split()
@@ -534,27 +353,43 @@ def cached_values_for(name):
             err))
 
 
-def initial_window_size(opts, cached_values):
-    w, h = opts.initial_window_width, opts.initial_window_height
+def initial_window_size_func(opts, cached_values):
+
     if 'window-size' in cached_values and opts.remember_window_size:
         ws = cached_values['window-size']
         try:
             w, h = map(int, ws)
+
+            def initial_window_size(*a):
+                return w, h
+            return initial_window_size
         except Exception:
             log_error('Invalid cached window size, ignoring')
-    return w, h
+
+    w, w_unit = opts.initial_window_width
+    h, h_unit = opts.initial_window_height
+
+    def get_window_size(cell_width, cell_height, dpi_x, dpi_y):
+        if w_unit == 'cells':
+            width = cell_width * w + (dpi_x / 72) * (opts.window_margin_width + opts.window_padding_width) + 1
+        else:
+            width = w
+        if h_unit == 'cells':
+            height = cell_height * h + (dpi_y / 72) * (opts.window_margin_width + opts.window_padding_width) + 1
+        else:
+            height = h
+        return width, height
+
+    return get_window_size
 
 
 def commented_out_default_config():
-    with open(default_config_path, encoding='utf-8', errors='replace') as f:
-        config = f.read()
-    lines = []
-    for line in config.splitlines():
-        if line.strip() and not line.startswith('#'):
+    ans = []
+    for line in as_conf_file(all_options.values()):
+        if line and line[0] != '#':
             line = '# ' + line
-        lines.append(line)
-    config = '\n'.join(lines)
-    return config
+        ans.append(line)
+    return '\n'.join(ans)
 
 
 def prepare_config_file_for_editing():
@@ -564,7 +399,7 @@ def prepare_config_file_for_editing():
             os.makedirs(d)
         except FileExistsError:
             pass
-        with open(defconf, 'w') as f:
+        with open(defconf, 'w', encoding='utf-8') as f:
             f.write(commented_out_default_config())
     return defconf
 
