@@ -4,19 +4,17 @@
 
 import os
 import pwd
-import shlex
 import sys
 from collections import namedtuple
 
 from .fast_data_types import set_boss as set_c_boss
 
 appname = 'kitty'
-version = (0, 10, 1)
+version = (0, 11, 0)
 str_version = '.'.join(map(str, version))
 _plat = sys.platform.lower()
 is_macos = 'darwin' in _plat
 base = os.path.dirname(os.path.abspath(__file__))
-editor = shlex.split(os.environ.get('EDITOR', 'vim'))
 
 
 ScreenGeometry = namedtuple('ScreenGeometry', 'xstart ystart xnum ynum dx dy')
