@@ -10,16 +10,18 @@ from contextlib import contextmanager
 from .borders import load_borders_program
 from .boss import Boss
 from .cli import create_opts, parse_args
-from .config import cached_values_for, initial_window_size
-from .constants import appname, glfw_path, is_macos, is_wayland, logo_data_file, config_dir
+from .config import cached_values_for, initial_window_size_func
+from .constants import (
+    appname, config_dir, glfw_path, is_macos, is_wayland, logo_data_file
+)
 from .fast_data_types import (
-    create_os_window, glfw_init, glfw_terminate, set_default_window_icon,
-    set_options, show_window
+    create_os_window, free_font_data, glfw_init, glfw_terminate,
+    set_default_window_icon, set_options, GLFW_MOD_SUPER
 )
 from .fonts.box_drawing import set_scale
+from .fonts.render import set_font_family
 from .utils import (
-    detach, end_startup_notification, init_startup_notification, log_error,
-    single_instance
+    detach, log_error, single_instance, startup_notification_handler
 )
 from .window import load_shader_programs
 
@@ -29,37 +31,71 @@ def load_all_shaders(semi_transparent=0):
     load_borders_program()
 
 
-def init_graphics():
+def init_graphics(debug_keyboard=False):
     glfw_module = 'cocoa' if is_macos else ('wayland' if is_wayland else 'x11')
+    if debug_keyboard:
+        os.environ['GLFW_DEBUG_KEYBOARD'] = '1'
     if not glfw_init(glfw_path(glfw_module)):
         raise SystemExit('GLFW initialization failed')
+    if debug_keyboard:
+        os.environ.pop('GLFW_DEBUG_KEYBOARD')
     return glfw_module
 
 
-def run_app(opts, args):
-    set_scale(opts.box_drawing_scale)
-    set_options(opts, is_wayland, args.debug_gl, args.debug_font_fallback)
+def prefer_cmd_shortcuts(x):
+    return x[0] == GLFW_MOD_SUPER
+
+
+def get_new_os_window_trigger(opts):
+    new_os_window_trigger = None
+    if is_macos:
+        new_os_window_shortcuts = []
+        for k, v in opts.keymap.items():
+            if v.func == 'new_os_window':
+                new_os_window_shortcuts.append(k)
+        if new_os_window_shortcuts:
+            from .fast_data_types import cocoa_set_new_window_trigger
+            new_os_window_shortcuts.sort(key=prefer_cmd_shortcuts, reverse=True)
+            for candidate in new_os_window_shortcuts:
+                if cocoa_set_new_window_trigger(*candidate):
+                    new_os_window_trigger = candidate
+                    break
+    return new_os_window_trigger
+
+
+def _run_app(opts, args):
+    new_os_window_trigger = get_new_os_window_trigger(opts)
     with cached_values_for(run_app.cached_values_name) as cached_values:
-        w, h = run_app.initial_window_size(opts, cached_values)
-        window_id = create_os_window(w, h, appname, args.name or args.cls or appname, args.cls or appname, load_all_shaders)
-        run_app.first_window_callback(opts, window_id)
-        startup_ctx = init_startup_notification(window_id)
-        show_window(window_id)
+        with startup_notification_handler(extra_callback=run_app.first_window_callback) as pre_show_callback:
+            window_id = create_os_window(
+                    run_app.initial_window_size_func(opts, cached_values),
+                    pre_show_callback,
+                    appname, args.name or args.cls or appname,
+                    args.cls or appname, load_all_shaders)
         if not is_wayland and not is_macos:  # no window icons on wayland
             with open(logo_data_file, 'rb') as f:
                 set_default_window_icon(f.read(), 256, 256)
-        boss = Boss(window_id, opts, args, cached_values)
+        boss = Boss(window_id, opts, args, cached_values, new_os_window_trigger)
         boss.start()
-        end_startup_notification(startup_ctx)
         try:
             boss.child_monitor.main_loop()
         finally:
             boss.destroy()
 
 
+def run_app(opts, args):
+    set_scale(opts.box_drawing_scale)
+    set_options(opts, is_wayland, args.debug_gl, args.debug_font_fallback)
+    set_font_family(opts)
+    try:
+        _run_app(opts, args)
+    finally:
+        free_font_data()  # must free font data before glfw/freetype/fontconfig/opengl etc are finalized
+
+
 run_app.cached_values_name = 'main'
-run_app.first_window_callback = lambda opts, window_id: None
-run_app.initial_window_size = initial_window_size
+run_app.first_window_callback = lambda window_handle: None
+run_app.initial_window_size_func = initial_window_size_func
 
 
 def ensure_osx_locale():
@@ -172,7 +208,9 @@ def _main():
             single_instance.socket.sendall(data)
             return
     opts = create_opts(args)
-    init_graphics()
+    if opts.editor != '.':
+        os.environ['EDITOR'] = opts.editor
+    init_graphics(args.debug_keyboard)
     try:
         with setup_profiling(args):
             # Avoid needing to launch threads to reap zombies
